﻿/**
 * @file
 * VuoCompilerGraph interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERGRAPH_H
#define VUOCOMPILERGRAPH_H

#include "VuoCompilerChain.hh"
#include "VuoCompilerLeaf.hh"
#include "VuoCompilerGeneratorAction.hh"
#include "VuoCompilerGraphvizParser.hh"

class VuoCompiler;

/**
 * Generates code for a graph. Creates a .bc file from a .vuo composition file.
 */
class VuoCompilerGraph
{
private:
	VuoCompilerGraphvizParser *parser;
	VuoCompiler *compiler;
	map<VuoCompilerGeneratorPort *, VuoCompilerNode *> nodeForGenerator;
	map<VuoCompilerGeneratorPort *, vector<VuoCompilerGeneratorEdge *> > generatorEdgesForGenerator;
	map<VuoCompilerGeneratorPort *, map<VuoCompilerNode *, vector<VuoCompilerGeneratorEdge *> > > generatorInEdgesForGeneratorAndNode;
	map<VuoCompilerGeneratorPort *, map<VuoCompilerNode *, vector<VuoCompilerPassiveEdge *> > > passiveInEdgesForGeneratorAndNode;
	map<VuoCompilerGeneratorPort *, map<VuoCompilerNode *, vector<VuoCompilerPassiveEdge *> > > passiveOutEdgesForGeneratorAndNode;
	map<VuoCompilerGeneratorPort *, vector<VuoCompilerNode *> > nodesForGenerator;
	map<VuoCompilerGeneratorPort *, set<VuoCompilerNode *> > loopEndNodesForGenerator;
	map<VuoCompilerGeneratorPort *, vector<VuoCompilerChain *> > chainsForGenerator;
	map<VuoCompilerGeneratorPort *, VuoCompilerGeneratorAction *> generatorObjectForGenerator;  // TODO: Refactor the above maps as member variables of VuoCompilerGeneratorAction
	vector<VuoCompilerLeaf *> leaves;
	Module *module;
	Function *setInputPortValueFunction;
	Function *getOutputPortValueFunction;
	Function *nodeExecutionEndedFunction;
	map<VuoCompilerNode *, GlobalVariable *> semaphoreVariableForNode;  // A semaphore to wait on while a node's event function is executing.
	bool debugMode;

	VuoCompilerGraph(void);
	void initialize(void);
	void makeEdgesForGenerator(VuoCompilerGeneratorPort *generator);
	void makeNodesForGenerator(VuoCompilerGeneratorPort *generator);
	void makeChainsForGenerator(VuoCompilerGeneratorPort *generator);
	void makeLeaves(void);
	void makeGeneratorObjects(void);
	void reportInfiniteLoop(VuoCompilerGeneratorPort *generator, map<VuoCompilerNode *, vector<VuoCompilerPassiveEdge *> > edgesForNode);
	bool isEdgeInInfiniteFeedbackLoop(VuoCompilerGeneratorPort *generator, VuoCompilerEdge *startEdge);
	bool edgeExists(VuoCompilerNode *fromNode, VuoCompilerNode *toNode, VuoCompilerGeneratorPort *generator);
	bool isDeadEnd(VuoCompilerNode *node, VuoCompilerGeneratorPort *generator);
	bool isScatter(VuoCompilerNode *node, VuoCompilerGeneratorPort *generator);
	bool isGather(VuoCompilerNode *node, VuoCompilerGeneratorPort *generator);
	void generateAllocation(void);
	void generateInitializationForReferenceCounts(BasicBlock *block);
	void generateFinalizationForReferenceCounts(BasicBlock *block);
	Function * generateVuoAllocFunction(void);
	Function * generateVuoRegisterFunction(void);
	Function * generateVuoRetainFunction(void);
	Function * generateVuoReleaseFunction(void);
	void generateSetupFunction(void);
	void generateCleanupFunction(void);
	void generateInitFunction(void);
	void generateFiniFunction(void);
	Function * generateGetOutputPortValueFunction(void);
	Function * generateSetInputPortValueFunction(void);
	void generateInitializationForInputPorts(BasicBlock *block);
	void generatePublishedPortGetters(void);
	void generateGetPublishedPortCountFunction(bool input);
	void generateGetPublishedPortNamesFunction(bool input);
	void generateGetPublishedPortTypesFunction(bool input);
	void generateFunctionReturningStringArray(string functionName, vector<string> stringValues);
	void generateGetPublishedPortConnectedIdentifierCount(bool input);
	void generateGetPublishedPortConnectedIdentifiers(bool input);
	void generateGeneratePublishedInputPortEventFunction(void);
	Function *generateNodeExecutionEndedFunction(void);
	Function * generateGeneratorFunctionHeader(VuoCompilerGeneratorPort *generator);  // TODO: Move to VuoCompilerGeneratorAction?
	void generateGeneratorFunctionBody(VuoCompilerGeneratorPort *generator);  // TODO: Move to VuoCompilerGeneratorAction?

	friend class TestVuoCompilerGraph;
	friend class TestVuoCompilerGraphExecution;

public:
	static VuoCompilerGraph * newCompilerGraphFromCompositionFile(const string &compositionPath, VuoCompiler *compiler);
	static VuoCompilerGraph * newCompilerGraphFromCompositionString(const string &composition, VuoCompiler * compiler);
	Module * generateBitcode(void);
	void setDebugMode(bool debugMode);
	VuoCompilerGraphvizParser * getParser(void);
};

#endif
