﻿/**
 * @file
 * VuoRendererNode interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUORENDERERNODE_HH
#define VUORENDERERNODE_HH

#include "VuoRendererItem.hh"
#include "VuoRendererPortList.hh"
#include "VuoRendererSignaler.hh"

#include "VuoBaseDetail.hh"
#include "VuoNode.hh"

class VuoCompilerNode;
class VuoCompilerNodeClass;
class VuoRendererPort;
class VuoNodeClass;
class VuoCable;

/**
 * Renders a node in a @c VuoRendererGraph.
 */
class VuoRendererNode : public VuoRendererItem, public VuoBaseDetail<VuoNode>
{
public:
	/**
	 * Specifies how this node is graphically represented.
	 */
	enum Type
	{
		node,
		detachedConstant,
		detachedDrawer,
		detachedTypecast
	};

	VuoRendererNode(VuoNode * baseNode, VuoRendererSignaler *signaler);

	QRectF boundingRect(void) const;
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
	void highlight(QPainter *painter);

	QRectF getNodeFrameRect(void) const;

	void setPortConnected(VuoRendererPort *rendererPort, bool isOutputPort, bool isConnected, bool hasData=false);

	void setProxyNode(VuoRendererNode * proxyNode);
	VuoRendererNode * getProxyNode(void) const;

	QVariant itemChange(GraphicsItemChange change, const QVariant &value);
	void updateGeometry(void);
	void updateConnectedCableGeometry(void);

	set<VuoCable *> getConnectedCables(void);
	set<VuoCable *> getConnectedInputCables(void);
	set<VuoCable *> getConnectedOutputCables(void);
	VuoRendererPortList * getInputPorts(void);
	VuoRendererPortList * getOutputPorts(void);
	void replaceInputPort(VuoRendererPort * oldPort, VuoRendererPort * newPort);
	void addInputPort(VuoRendererPort * newPort);

	// Drawing configuration
	static const qreal thickPenWidth; ///< Width, in pixels at 1:1 zoom, of the top and bottom node edges.  Many other drawing metrics are based on this value.
	static const qreal midPenWidth; ///< Width, in pixels at 1:1 zoom, of stateful indicator bars and typecast port edges.

private:
	VuoRendererNode *proxyNode; ///< The optional node that handles the rendering for this node.

	VuoRendererSignaler *signaler;

	// Drawing configuration
	static const qreal cornerRadius;
	static QFont nodeTitleFont;
	static QFont nodeClassFont;
	QRectF frameRect;

	// Node attributes affecting drawing
	enum Type nodeType;
	QString nodeTitle;
	QString nodeClass;
	bool nodeProvides;
	bool nodeConsumes;
	bool nodeIsStateful;
	bool nodeIsSubgraph;
	bool nodeAlwaysConducts;
	bool nodeIsAwake;
	bool nodeIsMissing;
	VuoRendererPort *receptorPort;
	VuoRendererPort *functionPort;
	VuoRendererPortList *inputPorts;
	VuoRendererPortList *outputPorts;

	// Internal methods
	void initWithNodeClass(VuoNodeClass *nodeClass);
	void setInputPorts(vector<VuoRendererPort *> inputPorts = vector<VuoRendererPort *>());
	void setOutputPorts(vector<VuoRendererPort *> outputPorts = vector<VuoRendererPort *>());
	void layoutPorts(void);

	void drawNodeFrame(QPainter *painter, QRectF bounds) const;
	//qreal getDrawerOffset(vector<VuoRendererPort *> ports, unsigned int portIndex) const;

	QPointF getPortPoint(unsigned int portIndex, bool isOutputPort) const;
};

#endif // VUORENDERERNODE_HH
