﻿/**
 * @file
 * VuoTelemetry interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOTELEMETRY_H
#define VUOTELEMETRY_H

/**
 * Control Requests, sent by the Vuo Editor.
 */
enum VuoControlRequest
{
	/**
	 * Request that the composition stop executing (without serialization).
	 */
	VuoControlRequestCompositionStop,

	/**
	 * Request that the composition pause execution (without serialization).
	 */
	VuoControlRequestCompositionPause,

	/**
	 * Request that the composition unpause execution.
	 */
	VuoControlRequestCompositionUnpause,

	/**
	 * Request that the composition serialize its state and stop executing.
	 */
	VuoControlRequestCompositionHibernate,

	/**
	 * Request that the input port be set to the given value (converted to the port's type).
	 *
	 * Includes data message-parts:
	 *		@arg @c char *portIdentifier;
	 *		@arg @c char *valueAsString;
	 */
	VuoControlRequestPortValueModify,

	/**
	 * Request that the output port's value be looked up and returned.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *portIdentifier;
	 */
	VuoControlRequestPortValueRetrieve,

	/**
	 * Request that the published input ports' names be looked up and returned.
	 */
	VuoControlRequestPublishedInputPortNamesRetrieve,

	/**
	 * Request that the published output ports' names be looked up and returned.
	 */
	VuoControlRequestPublishedOutputPortNamesRetrieve,

	/**
	 * Request that the published input ports' types be looked up and returned.
	 */
	VuoControlRequestPublishedInputPortTypesRetrieve,

	/**
	 * Request that the published output ports' types be looked up and returned.
	 */
	VuoControlRequestPublishedOutputPortTypesRetrieve,

	/**
	 * Request that the published input port's connected unpublished ports be looked up and returned.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *name
	 */
	VuoControlRequestPublishedInputPortConnectedIdentifiersRetrieve,

	/**
	 * Request that the published output port's connected unpublished ports be looked up and returned.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *name
	 */
	VuoControlRequestPublishedOutputPortConnectedIdentifiersRetrieve,

	/**
	 * Request that an event be generated and transmitted through the published input port.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *name
	 */
	VuoControlRequestPublishedInputPortGenerateEvent
};

/**
 * Control Replies, sent by the built composition.
 */
enum VuoControlReply
{
	/**
	 * Composition shutdown has been initiated.
	 */
	VuoControlReplyCompositionStopping,

	/**
	 * Composition has been paused.
	 */
	VuoControlReplyCompositionPaused,

	/**
	 * Composition has been unpaused.
	 */
	VuoControlReplyCompositionUnpaused,

	/**
	 * Hibernation is complete.
	 *
	 * At this point any composition outputs remain open but frozen, waiting for the Vuo Editor to launch and unserialize the successor process.  When that's complete, the Vuo Editor should send VuoControlCompositionStop to this process.
	 *
	 * Includes data message-parts: (serialized data, to be determined)
	 */
	VuoControlReplyCompositionHibernated,

	/**
	 * The input port's value has been set.
	 */
	VuoControlReplyPortValueModified,

	/**
	 * A string representation of the output port's value has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *valueAsString;
	 */
	VuoControlReplyPortValueRetrieved,

	/**
	 * The list of published input ports' names has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg char *name0
	 *		@arg char *name1
	 *		@arg ...
	 */
	VuoControlReplyPublishedInputPortNamesRetrieved,

	/**
	 * The list of published output ports' names has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg char *name0
	 *		@arg char *name1
	 *		@arg ...
	 */
	VuoControlReplyPublishedOutputPortNamesRetrieved,

	/**
	 * The list of published input ports' types, as string representations, has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg char *type0
	 *		@arg char *type1
	 *		@arg ...
	 */
	VuoControlReplyPublishedInputPortTypesRetrieved,

	/**
	 * The list of published output ports' types, as string representations, has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg char *type0
	 *		@arg char *type1
	 *		@arg ...
	 */
	VuoControlReplyPublishedOutputPortTypesRetrieved,

	/**
	 * The list of published input ports' connected unpublished ports has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg char *identifier0
	 *		@arg char *identifier1
	 *		@arg ...
	 */
	VuoControlReplyPublishedInputPortConnectedIdentifiersRetrieved,

	/**
	 * The list of published output ports' connected unpublished ports has been retrieved.
	 *
	 * Includes data message-parts:
	 *		@arg char *identifier0
	 *		@arg char *identifier1
	 *		@arg ...
	 */
	VuoControlReplyPublishedOutputPortConnectedIdentifiersRetrieved,

	/**
	 * An event has been generated and transmitted through the published input port.
	 */
	VuoControlReplyPublishedInputPortGeneratedEvent
};

/**
 * Types of published telemetry data.
 */
enum VuoTelemetry
{
	/**
	 * General information about the composition process, published every half-second.
	 *
	 * Includes data message-parts:
	 *		@arg @c unsigned long utime;
	 *		@arg @c unsigned long stime;
	 */
	VuoTelemetryStats,

	/**
	 * Published just prior to calling each node's nodeEvent/nodeInstanceEvent function.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *nodeIdentifier;
	 */
	VuoTelemetryNodeExecutionStarted,

	/**
	 * Published just after each node's nodeEvent/nodeInstanceEvent function returns.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *nodeIdentifier;
	 */
	VuoTelemetryNodeExecutionFinished,

	/**
	 * Published just after each node's nodeEvent/nodeInstanceEvent function returns,
	 * for nodes that have at least one output port that carries data.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *portIdentifier0;
	 *		@arg @c char *portValue0;
	 *		@arg @c char *portIdentifier0;
	 *		@arg @c char *portValue1;
	 *		@arg @c ...
	 */
	VuoTelemetryPortValuesChanged,

	/**
	 * Published when an event is conducted along an edge.
	 *
	 * Includes data message-parts:
	 *		@arg @c char *edgeIdentifier;
	 */
	VuoTelemetryEdgeHit
};


#ifdef __cplusplus
extern "C" {
#endif

char * copyStringFromMessage(zmq_msg_t *message);
void initMessageWithString(zmq_msg_t *message, const char *string);
char * receiveAndCopyString(void *socket);
unsigned long receiveUnsignedLong(void *socket);

#ifdef __cplusplus
}
#endif

#endif
